
# q-expansion of newform 8034.2.a.c, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8034
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [1], [1], [3], [2], [-1], [0], [4], [0], [7], [-4], [2], [-6], [2], [13], [2], [-5], [4], [-1], [0], [3], [-4], [-1], [-8], [14], [14], [-1], [18], [2], [19], [-20], [2], [13], [-4], [-18], [2], [-13], [-2], [-18], [22], [-4], [-6], [22], [17], [-1], [9], [10], [-27], [24], [10], [6], [-6], [2], [1], [13], [-10], [9], [30], [-31], [-10], [-6], [-30], [-28], [-35], [-17], [18], [17], [-14], [-28], [28], [-24], [-18], [16], [-4], [1], [21], [-14], [-4], [33], [-34], [10], [37], [-10], [26], [13], [33], [42], [9], [24], [-34], [-14], [-15], [12], [12], [-16], [-37], [18], [21], [9], [7], [-1], [18], [-1], [-33], [21], [-15], [41], [-30], [-12], [-44], [28], [26], [-8], [14], [-40], [-24], [-50], [31], [-12], [10], [-42], [-37], [13], [14], [-11], [9], [-11], [-28], [-7], [22], [10], [15], [-24], [4], [48], [13], [14], [14], [-7], [39], [19], [8], [-5], [20], [23], [14], [15], [-18], [40], [20], [4], [46], [20], [-29], [-27], [38], [-43], [-51], [-58], [18], [-36], [-2], [18], [-51], [-54], [-18], [56], [-23], [22], [-58], [5], [10], [45], [-42], [-16], [0], [16], [10], [12], [-1], [44], [62], [40], [52], [47], [27], [-27], [4], [29], [-46], [-30], [-63], [-4], [-18], [47], [30], [52], [3], [-10], [34], [27], [-16], [-17], [55], [45], [-51], [42], [21], [-10], [-37], [-67], [-60], [-31], [66], [-64], [-10], [40], [-26], [-42], [12], [69], [-5], [-64], [-32], [-19], [-29], [-13], [40], [-8], [-9], [-54], [-31], [59], [-45], [-65], [48], [-14], [30], [-63], [-30], [-31], [-70], [49], [65], [-1], [-16], [-44], [-31], [-36], [76], [38], [1], [-43], [42], [-49], [24], [-30], [70], [-59], [-34], [-26], [-72], [10], [-66], [73], [-62], [-8], [-20], [-4], [-38], [-22], [51], [-10], [-6], [7], [-44], [-52], [-69], [-20], [-80], [1], [-42], [30], [27], [14], [57], [-39], [-72], [-36], [75], [-72], [69], [-27], [-42], [-46], [66], [48], [-54], [-15], [66], [-63], [-3], [-9], [-2], [-32], [-56], [-52], [-56], [12], [-38], [-86], [28], [-3], [-63], [27], [53], [24], [-21], [-22], [-49], [-46], [-26], [-34], [-5], [57], [6], [-30], [24], [-42], [-29], [25], [12], [-80], [2], [-39], [9], [54], [27], [42], [-8], [-22], [54], [-39], [-1], [-74], [64], [-17], [-8], [27], [-47], [-50], [-24], [-55], [52], [35], [-18], [-6], [-56], [-87], [-35], [-7], [-56], [66], [-74], [-40], [90], [-37], [92], [71], [-86], [-34], [-33], [20], [-26], [13], [59], [47], [-54], [-54], [-8], [70], [-93], [-1], [11], [-37], [21], [34], [10], [7], [33], [-34], [-20], [72], [-77], [80], [36], [63], [-31], [40], [-64], [16], [-46], [28], [20], [-48], [95], [18], [-40], [68], [47], [-82], [-37], [-1], [-30], [-13], [-14], [72], [-32], [-9], [57], [-62], [-27], [-45], [-32], [-23], [6], [4], [50], [-1], [24]]
